/**  A small class to show information about a zip archive, and
     to unzip it also.

      To Do :- 1) Make sure that the user confirms a deletion.

               2) More options. [Definately a filename -> lowercase]
   @version 1
   @author Steve Kemp
 */


import java.io.*;
import java.util.zip.*;
import java.util.*;

public class unzip
{
  public static void main(String args[])
  {
    boolean displayInfo = false;

    if ( args.length < 1 )
    { System.out.println( "Usage: unzip [-l] file.zip" );
      return;
    }

    for ( int i = 0; i < args.length; i++ )
    { if ( args[i].trim().equals( "-l" ) )
        displayInfo = true;
    }

    if ( displayInfo )
    { for ( int j = 0; j < args.length; j++ )
      {  if ( !args[j].trim().equals("-l") )
           listFiles( args[j] );
      }
      return;
    }
    else
    { for ( int k = 0; k < args.length; k++ )
        if ( !args[ k ].trim().equals( "-l" ) )
           unzip( args[ k ] );
    }
  }



/**  This method lists information about the files contained in a
    zip archive.
    @param filename The name of the zip archive to investigate
 */
  public static void listFiles( String filename )
  {
    int numberOfFiles = 0;
   
    try
    {
      // Create an object to represent the zip file
      ZipFile zipFile = new ZipFile( filename );

      // Print out the name of the file
      System.out.println( zipFile.getName() + " contains:-" );

      // And now start to enumerate through the entries
      Enumeration entries = zipFile.entries();

      // Print out a heading
      System.out.println( "Size\t\t% Compressed\tName" );

      // Loop through all the entries
      while ( entries.hasMoreElements() )
      { 
        // Get an entry
        ZipEntry zipEntry = ( ZipEntry )entries.nextElement();

        // Print out its information
        System.out.println( zipEntry.getSize() + "\t\t" + 
          getCompressionRatio( zipEntry.getSize(), 
          zipEntry.getCompressedSize() ) + "\t\t" +
          zipEntry.getName() );

        // Increase the count of files.
        numberOfFiles++;
      }

      // Close the zip file
      zipFile.close();
 
      // Now print the number of files in the archive
      System.out.println( "\t\t" + numberOfFiles + " entries" );
    }
    catch (IOException e)
    { System.out.println( "Error listing contents of the zip file" );
    }
  }


/** A simple method to return the compression ratio of a file
    @param orig The original size of the file
    @param compressed The compressed size of the file
    @return The percentage compression achieved
 */
  public static int getCompressionRatio( long orig, long compressed )
  { int result;
    if ( compressed != 0L )
      result = ( int )( orig / compressed ) * 100;
    else
      result = 0;
    return result;
  }


  /** Unzip all of the files in a directory, making sure that
      the output appears in the correct directory structure.
      @param filename The name of the zip file to unzip
    */
  public static void unzip(String filename)
  { 
    try
    { 
      System.out.println("Unzipping archive " + filename );

      // Create an object to represent the zip file
      ZipFile zipFile = new ZipFile( filename );
        
      Enumeration entries = zipFile.entries();
       
      // Want to loop through the file, processing each entry
      while ( entries.hasMoreElements() )
      { 

        // Get the entry
        ZipEntry zipEntry = ( ZipEntry )entries.nextElement();

        // Create the output file in the correct place.
        File outputFile = makeOutputFile( zipEntry.getName(), zipEntry.isDirectory() );

        // If we have just created a directory then skip this
        // part of the loop.
        if ( outputFile.isDirectory() )
          continue;

        copy( zipFile.getInputStream( zipEntry ), outputFile );

      }

      zipFile.close();
    }
    catch (IOException e)
    { System.out.println("Error opening unzipping file\n" + e );
    }
  }


  /**   Construct a file that will contain the output of the
      zipped file.
        This really boils down to making sure that any needed
      directorys are present.
      @param name This is the name of the file to return
   */
  private static File makeOutputFile( String name, boolean isdir )
  {
   
     String currentDir = System.getProperty( "user.dir" );
     String fileSep = System.getProperty( "file.separator" );

     // Remove a trailing slash - if any
     if ( name.endsWith( fileSep ) )
       name = name.substring( 0, name.length() - 1 );

     // Create the file
     File output = new File( currentDir, name );
     
     // Now see where the last "/" or "\" is, this is so that
     // we can make sure that if there is a directory portion in
     // the file it is made
     int slash = name.lastIndexOf( fileSep );
     
     // A slash, so we might have to make the directory
     if ( slash != -1 || isdir )
     { 
       // Get the name of the dir
       String dirName = isdir ? name : name.substring( 0, slash );

       // Represtent it, then see if it already exists
       File dir = new File( currentDir, dirName );

       // If it exists return the file we created earlier
       if ( dir.exists() )
         return( output );

       // Otherwise try to create the directory hierarch pointing here
       if ( !dir.mkdirs( ) )
         System.out.println( "Could not create output directory " + dirName );

       // Return
       return( output );
           
     }
     
     return( output );

  }


  /** Copy bytes from the zip entry to the file object that should
      have been created, this effectively unzips the file.
      @param source A stream to read the bytes from
      @param out The file to write them too.
   */
  public static void copy( InputStream source, File out )
  {
    try
    { FileOutputStream destination = new FileOutputStream( out );
      byte buffer[] = new byte[1024];
      while(true)
      {
        int bytes_read = source.read( buffer );
        if (bytes_read == -1)
          break;
        destination.write( buffer, 0, bytes_read );
      }
      destination.close();
    }
    catch( IOException e )
    { 
    }
  }
}

